﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Data.OleDb;
using System.Data.Common;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows;
using Microsoft.Data;
using Microsoft.Data.ConnectionUI;



namespace CubeExplorerUI
{
    public struct StoredProcedure
    {
        public string nom;
        public int object_id;
    }

    public class CarregadorDades
    {
        List<string> lsTaules = null;
        List<string> lsVistes = null;
        Dictionary <string,StoredProcedure> dcSp = null;

        string name;
        bool connectat;
        string provider;
        string strconnexio;         // cadena de connexió
      
        SqlConnection cnnSql;
        OleDbConnection cnnOleDb;

        public string Connexio
        {
            get { return strconnexio; }
        }
        
        public CarregadorDades()
        {
            connectat = false;
        }

        public string Provider
        {
            get { return (provider); }
        }
        public bool Connectat
        {
            get { return (connectat); }
        }
       
        public List<string> Taules
        {
            get { return (lsTaules); }
        }

        public List<string> Vistes
        {
            get { return (lsVistes); }
        }

        public Dictionary<string,StoredProcedure> SP
        {
            get { return (dcSp); }
        }
        
        
        private void CarregarCatalegSQL()
        {

            // taules
            string sql = "select name from sys.tables";

            SqlCommand command = new SqlCommand(sql, cnnSql);
            SqlDataReader reader = command.ExecuteReader();

            lsTaules = new List<string>(); 
            while (reader.Read())
            {
                 lsTaules.Add(reader[0].ToString()); 
            }

            reader.Close();
            command.Dispose();

            // vistes
            sql = "select name from sys.views";

            command = new SqlCommand(sql, cnnSql);
            reader = command.ExecuteReader();

            lsVistes = new List<string>(); 
            while (reader.Read())
            {
                lsVistes.Add(reader[0].ToString());
            }

            reader.Close();
            command.Dispose();

            // stored procedures
            sql = "select name,object_id from sys.procedures";

            command = new SqlCommand(sql, cnnSql);
            reader = command.ExecuteReader();

            StoredProcedure sp;

            dcSp = new Dictionary <string,StoredProcedure>(); 
            while (reader.Read())
            {
                sp.nom = reader[0].ToString();
                sp.object_id = Convert.ToInt32 (reader[1]);
                // indexem per nom per poder buscar els paràmetres
                dcSp.Add(sp.nom,sp);
            }

            reader.Close();
            command.Dispose();

           
        }

        
        private void CarregarCatalegSQLOLE()
        {

            // taules
            string sql = "select name from sys.tables";


            OleDbCommand command = new OleDbCommand(sql, cnnOleDb);
            OleDbDataReader  reader = command.ExecuteReader();

            lsTaules = new List<string>();
            while (reader.Read())
            {
                lsTaules.Add(reader[0].ToString());
            }

            reader.Close();
            command.Dispose();

            // vistes
            sql = "select name from sys.views";

            command = new OleDbCommand(sql, cnnOleDb);
            reader = command.ExecuteReader();

            lsVistes = new List<string>();
            while (reader.Read())
            {
                lsVistes.Add(reader[0].ToString());
            }

            reader.Close();
            command.Dispose();

            // stored procedures
            sql = "select name,object_id from sys.procedures";

            command = new OleDbCommand(sql, cnnOleDb);
            reader = command.ExecuteReader();

            StoredProcedure sp;

            dcSp = new Dictionary<string, StoredProcedure>();
            while (reader.Read())
            {
                sp.nom = reader[0].ToString();
                sp.object_id = Convert.ToInt32(reader[1]);
                // indexem per nom per poder buscar els paràmetres
                dcSp.Add(sp.nom, sp);
            }


            reader.Close();
            command.Dispose();
        }


        public SqlDbType ConvertType(string type) {

            switch (type)
            {
                case "image":
                    return SqlDbType.Image;
                    
                case "text":
                    return SqlDbType.Text;
                    
                case "uniqueidentifier":
                    return SqlDbType.UniqueIdentifier;

                    
                case "date":
                    return SqlDbType.Date;

                    
                case "time":
                    return SqlDbType.Time;
                    

                case "datetime2":
                    return SqlDbType.DateTime2;
                    

                case "datetimeoffset":
                    return SqlDbType.DateTimeOffset;
                    
                case "tinyint":
                    return SqlDbType.TinyInt;
                    
                case "smallint":
                    return SqlDbType.SmallInt;
                    
                case "int":
                    return SqlDbType.Int;
                    
                case "smalldatetime":
                    return SqlDbType.Int;
                    


                case "real":
                    return SqlDbType.Real;
                    

                case "money":
                    return SqlDbType.Money;
                    

                case "datetime":
                    return SqlDbType.DateTime;
                    

                case "float":
                    return SqlDbType.Float;
                    

                case "sql_variant":
                    return SqlDbType.Variant;
                    

                case "ntext":
                    return SqlDbType.NText;
                    

                case "bit":
                    return SqlDbType.Bit;
                    

                case "decimal":
                    return SqlDbType.Decimal;
                    

                case "numeric":
                    return SqlDbType.Decimal;
                    

                case "smallmoney":
                    return SqlDbType.SmallMoney;
                    

                case "bigint":
                    return SqlDbType.BigInt;
                    

                case "hierarchyid":
                    return SqlDbType.UniqueIdentifier;
                    

                case "geometry":
                    return SqlDbType.Variant;
                    

                case "geography":
                    return SqlDbType.Variant;
                    

                case "varbinary":
                    return SqlDbType.VarBinary;
                    

                case "varchar":
                    return SqlDbType.VarChar;
                    

                case "binary":
                    return SqlDbType.Binary;
                    

                case "char":
                    return SqlDbType.Char;
                    

                case "timestamp":
                    return SqlDbType.Timestamp;
                    

                case "nvarchar":
                    return SqlDbType.NVarChar;
                    

                case "nchar":
                    return SqlDbType.NChar;
                    

                case "xml":
                    return SqlDbType.Xml;
                    

                case "sysname":
                    return SqlDbType.Variant;
                    
                default:
                    return 0;
                    
            }
       
        }


        public OleDbType ConvertTypeOLE(string type)
        {

            switch (type)
            {
                case "image":
                    return OleDbType.VarBinary;

                case "text":
                    return OleDbType.VarChar;

                case "uniqueidentifier":
                    return OleDbType.Guid;


                case "date":
                    return OleDbType.Date;


                case "time":
                    return OleDbType.DBTime; 


                case "datetime2":
                    return OleDbType.DBTimeStamp;


                case "datetimeoffset":
                    return OleDbType.DBTimeStamp;

                case "tinyint":
                    return OleDbType.TinyInt;

                case "smallint":
                    return OleDbType.SmallInt;

                case "int":
                    return OleDbType.Integer;

                case "smalldatetime":
                    return OleDbType.Date;



                case "real":
                    return OleDbType.Double;


                case "money":
                    return OleDbType.Currency;


                case "datetime":
                    return OleDbType.DBTimeStamp;


                case "float":
                    return OleDbType.Single;


                case "sql_variant":
                    return OleDbType.Variant;


                case "ntext":
                    return OleDbType.VarChar;


                case "bit":
                    return OleDbType.Boolean;


                case "decimal":
                    return OleDbType.Decimal;


                case "numeric":
                    return OleDbType.Decimal;


                case "smallmoney":
                    return OleDbType.Currency;


                case "bigint":
                    return OleDbType.BigInt;


                case "hierarchyid":
                    return OleDbType.Guid;


                case "geometry":
                    return OleDbType.Variant;


                case "geography":
                    return OleDbType.Variant;


                case "varbinary":
                    return OleDbType.VarBinary;


                case "varchar":
                    return OleDbType.VarChar;


                case "binary":
                    return OleDbType.Binary;


                case "char":
                    return OleDbType.Char;


                case "timestamp":
                    return OleDbType.DBTimeStamp;


                case "nvarchar":
                    return OleDbType.VarChar;


                case "nchar":
                    return OleDbType.VarChar;


                case "xml":
                    return OleDbType.VarWChar;


                case "sysname":
                    return OleDbType.VarChar;

                default:
                    return 0;

            }

        }

        public DataTable ExecuteSQL(string sql)
        {
            if (provider == "System.Data.SqlClient")
            {
                DataTable dt = new DataTable();

                SqlDataAdapter da = new SqlDataAdapter(sql, cnnSql);
                
                da.Fill(dt);

                return (dt);  
            }
            else if (provider == "System.Data.OleDb")
            {
                DataTable dt = new DataTable();

                OleDbDataAdapter da = new OleDbDataAdapter(sql, cnnOleDb);

                da.Fill(dt);

              
                return (dt); 
            }
            else
            {
                return null;
            }

        }



        public SqlParameterCollection GetSPParametersSQL(int object_id)
        {
            string sql = "select sp.name,sp.system_type_id,sp.max_length,st.name" +  
                         " from sys.parameters sp " +
                         " inner join sys.types st on sp.system_type_id = st.system_type_id " +
                         " and sp.user_type_id = st.user_type_id " +
                         " where sp.object_id = " + object_id.ToString();

            SqlParameterCollection parameters = new SqlCommand().Parameters; 

            
            SqlCommand  command = new SqlCommand(sql, cnnSql);
            SqlDataReader  reader = command.ExecuteReader();
            SqlDbType dbType;

            while (reader.Read())
            {
                dbType = ConvertType(reader[3].ToString());
                SqlParameter ps = new SqlParameter(reader[0].ToString(), dbType);
                parameters.Add(ps);

            }
            reader.Close();
            return parameters;
        }


        public OleDbParameterCollection GetSPParametersSQLOLE(int object_id)
        {

            string sql = "select sp.name,sp.system_type_id,sp.max_length,st.name" +
                        " from sys.parameters sp " +
                        " inner join sys.types st on sp.system_type_id = st.system_type_id " +
                        " and sp.user_type_id = st.user_type_id " +
                        " where sp.object_id = " + object_id.ToString();
            
            OleDbParameterCollection parameters = new OleDbCommand().Parameters;


            OleDbCommand command = new OleDbCommand(sql, cnnOleDb);
            OleDbDataReader reader = command.ExecuteReader();
            OleDbType dbType;


            while (reader.Read())
            {
                dbType = ConvertTypeOLE(reader[3].ToString());
                OleDbParameter ps = new OleDbParameter(reader[0].ToString(), dbType);
                parameters.Add(ps);
            }
            return parameters;
        }


        public DbParameterCollection GetSpParameters(int object_id)
        {
            if (provider == "System.Data.SqlClient")
            {
                return GetSPParametersSQL(object_id);
            }
            else if (provider == "System.Data.OleDb")
            {
                return GetSPParametersSQLOLE(object_id); 
            }
            else
                return null;
        }


        public DbParameterCollection GetSpParameters(string name)
        {
            // Obtenir el object_id corresponent al nom del procedure
            string sql = "select object_id from sys.procedures where name= '" + name + "'";

            DataTable dt = ExecuteSQL(sql);

            if (dt.Rows.Count > 0) {
                int object_id = Convert.ToInt32 (dt.Rows[0][0]);
                return GetSpParameters(object_id);
            } else {
               return null;
            } 
        }


        private void CarregarCatalegAccess()
        {
            try
            {
                string[] restrictions = new string[4];

                restrictions[3] = "TABLE";
 
                DataTable table = cnnOleDb.GetOleDbSchemaTable(OleDbSchemaGuid.Tables, restrictions); 

                lsTaules = new List<string>();
              
                foreach (DataRow row in table.Rows)
                {
                    lsTaules.Add(row["TABLE_NAME"].ToString());
              
                }

          
                DataTable queries = cnnOleDb.GetOleDbSchemaTable(OleDbSchemaGuid.Procedures , null); 

                lsVistes = new List<string>();
               
                foreach (DataRow row in queries.Rows)
                {
                   if  (row["PROCEDURE_NAME"].ToString().IndexOf ('~') == -1)
                        lsVistes.Add(row["PROCEDURE_NAME"].ToString());

                }



               
            }
            catch (OleDbException e)
            {
                
            }

        }

        public DataSet ObtenirDades(string element, ref string sql)
        {
            DataSet ds = null;
            // Executar la consulta
            if (provider == "System.Data.SqlClient")
            {
                sql = "Select * from [" + element + "]";
                SqlDataAdapter da = new SqlDataAdapter(sql, cnnSql);
                ds = new DataSet();
                da.Fill(ds);
            }
            else if (provider == "System.Data.OleDb")
            {
                sql = "Select * from [" + element + "]";
                OleDbDataAdapter da = new OleDbDataAdapter(sql, cnnOleDb);
                ds = new DataSet();
                da.Fill(ds);

            }
            return ds;
        }

        public DataSet ObtenirDades(string sql)
        {
            DataSet ds = null;
            // Executar la consulta
            if (provider == "System.Data.SqlClient")
            {
                SqlDataAdapter da = new SqlDataAdapter(sql, cnnSql);
                ds = new DataSet();
                da.Fill(ds);
            }
            else if (provider == "System.Data.OleDb")
            {
                OleDbDataAdapter da = new OleDbDataAdapter(sql, cnnOleDb);
                ds = new DataSet();
                da.Fill(ds);

            }
            return ds;
        }


        public DataSet ObtenirDades(string element, DbParameterCollection parametres)
        {
            DataSet ds = null;
            // Executar la consulta
            if (provider == "System.Data.SqlClient")
            {
                SqlCommand cmd = new SqlCommand(element, cnnSql);
                cmd.CommandType = CommandType.StoredProcedure;
                
                foreach (DbParameter p in parametres)
                {
                    SqlParameter p2 = new SqlParameter(p.ParameterName,p.Value);
                   
                    cmd.Parameters.Add(p2);
                }

                SqlDataAdapter da = new SqlDataAdapter(cmd);
                ds = new DataSet();
                da.Fill(ds);
 
            }
            else if (provider == "System.Data.OleDb")
            {
                OleDbCommand cmd = new OleDbCommand(element, cnnOleDb);
                cmd.CommandType = CommandType.StoredProcedure;
                foreach (DbParameter p in parametres)
                {
                    cmd.Parameters.Add(p);
                }

                OleDbDataAdapter da = new OleDbDataAdapter(cmd);
                ds = new DataSet();
                da.Fill(ds);
 
            }
            return ds;
            
        }

        public bool Connectar(string connectionString)
        {
            bool connectat=false;
            try
            {
                // Utilitzem aquesta classe per identificar els paràmetres de connexió
                DbConnectionStringBuilder builder = new DbConnectionStringBuilder();
                builder.ConnectionString = connectionString;

                // si s'ha definit el atribut provider la connexió és per OLEDB                
                if (builder.ContainsKey("provider"))
                {
                    provider =  (string)builder["provider"];
                    if (provider.Contains("Microsoft."))
                    {
                        // proveïdors Excel/Access 2003, 2007
                        name = "MicrosoftAccess";
                    } else {
                        name = "MicrosoftSqlServer";
                    }
                    provider = "System.Data.OleDb";

                } else {
                    name = "MicrosoftSqlServer";
                    provider = "System.Data.SqlClient";
                }
                
                
                strconnexio = connectionString;
                
                switch (name)
                {

                    case "MicrosoftAccess":
                        // Connexió per OLE DB 
                        cnnOleDb = new OleDbConnection(connectionString);
                   
                        cnnOleDb.Open();
                        connectat = true;
                   
                        break;

                    case "MicrosoftSqlServer":
                        
                        if (provider == "System.Data.OleDb")
                        {
                            // Connexió per OLE DB
                            cnnOleDb = new OleDbConnection(connectionString);
                     
                            cnnOleDb.Open();
                            connectat = true;
               
                        }
                        else if (provider == "System.Data.SqlClient")
                        {
                            // Connexió per proveïdor .NET
                            cnnSql = new SqlConnection(connectionString);
                          
                            cnnSql.Open();
                            connectat = true;
                          
                        }
                        break;
                    default:
                        // Connexions no suportades
                        MessageBox.Show("Connexió no suportada.", "CubeExplorer");
                        break;
                }



                return connectat;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Source + ": " + e.Message, "CubeExplorer");

                connectat = false;
                return connectat;
            }
        }


        public bool CarregarCataleg()
        {
            connectat = false;
            switch (name)
                    {
                       
                        case "MicrosoftAccess":
                            // Connexió per OLE DB 
                            cnnOleDb = new OleDbConnection(strconnexio);
                            

                            cnnOleDb.Open();
                            connectat = true;
                            CarregarCatalegAccess();
              
                            break;

                        case "MicrosoftSqlServer":
                                                  
                       
                            if (provider == "System.Data.OleDb")
                            {
                                // Connexió per OLE DB
                                cnnOleDb = new OleDbConnection(strconnexio);
                              

                                cnnOleDb.Open();
                                connectat = true;
                                CarregarCatalegSQLOLE();
                                

                                break;
                                
                            }
                            else if (provider == "System.Data.SqlClient")
                            {
                                // Connexió per proveïdor .NET
  
                                cnnSql = new SqlConnection(strconnexio);
                               

                                cnnSql.Open();
                                connectat = true;
                                CarregarCatalegSQL();
                                
                            }
                            break;
                        default:
                            // Connexions no suportades
                            MessageBox.Show("Connexió no suportada.", "CubeExplorer");
                            break;
                   }
            return connectat; 
        }
       
        
        
        public bool Connectar()
        {
            try
            {
                DataConnectionDialog objDataConnectionDialog = new DataConnectionDialog();

                DataSource.AddStandardDataSources(objDataConnectionDialog);

                if (DataConnectionDialog.Show(objDataConnectionDialog) == System.Windows.Forms.DialogResult.OK)
                {
                    name = objDataConnectionDialog.SelectedDataSource.Name;
                    provider = objDataConnectionDialog.SelectedDataProvider.Name;
                    strconnexio = objDataConnectionDialog.ConnectionString;

                    connectat = CarregarCataleg();

                    objDataConnectionDialog.Dispose();
                }
                return connectat;
               
            } catch (Exception e) {
                MessageBox.Show  (e.Source + ": " + e.Message,"CubeExplorer"); 
                
                connectat = false;
                return connectat; 
            }

        }

        public void Close()
        {
            if (cnnOleDb != null && cnnOleDb.State == ConnectionState.Open) 
                cnnOleDb.Close();
            if (cnnSql != null && cnnSql.State == ConnectionState.Open) 
                cnnSql.Close(); 
        }
    }

    
}
