﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using DevExpress.Xpf.Core;
using System.Data;
using DevExpress.Xpf.PivotGrid;


namespace CubeExplorerUI
{
    /// <summary>
    /// Interaction logic for ConstruirCubs.xaml
    /// </summary>
    public partial class ConstruirCubs : Window
    {
        // DataSet
        DataSet ds;
        
        // Propietats del cub
        public string carpeta { get; set; }
        public string rutacompleta { get; set; }
        public string nom { get; set;  }
        public string descripcio { get; set; }
        public int tipus { get; set; }
       
        // flag guardat
        bool guardat;

                     
        public ConstruirCubs()
        {
            InitializeComponent();
            carpeta = "";
            nom = "";
            descripcio = "";
            tipus = 1;
            guardat = false;
        }

        public void Construir(string carpeta)
        {
            // Cub desconnectat, les dades es carreguen directament de disc
            this.carpeta = carpeta;
           
            pivotGridControl1.RestorePivotGridFromFile(rutacompleta);
        }

        
        public void Construir (List<FormatCamp> llista, DataSet ds, string carpeta) {

            this.ds = ds;
            this.carpeta = carpeta;
            
            pivotGridControl1.DataSource = ds.Tables[0];
            pivotGridControl1.BeginUpdate();

            // Camps que es volen crear segons la sel.lecció del usuari
            PivotGridField[] fields = new PivotGridField [llista.Count];
            int i=0;
            // Construcció del cub
            foreach (FormatCamp fc in llista)
            {
                fields[i] = new PivotGridField();
                fields[i].FieldName = fc.Camp;
                fields[i].Caption = fc.Títol;
                
                switch (fc.Area) {
                    case "Columnes" : 
                        fields[i].Area = FieldArea.ColumnArea;
                    break;

                    case "Files":
                        fields[i].Area = FieldArea.RowArea;
                    break;

                    case "Filtres":
                        fields[i].Area = FieldArea.FilterArea;
                    break;
                    case "Dades": 
                        fields[i].Area = FieldArea.DataArea;
                    break;
                }

                switch (fc.Format)
                {
                    case "Moneda":
                        fields[i].CellFormat = "c";
                    break;
                    case "Moneda amb 4 decimals":
                        fields[i].CellFormat = "c4";

                    break;
                    case "Decimal":
                        fields[i].CellFormat = "d";
                    break;
                    case "Científic":
                        fields[i].CellFormat = "e";
                    break;
                    
                    case "Fixe":
                        fields[i].CellFormat = "f3";
                    break;
                    case "General":
                        fields[i].CellFormat = "g";
                    break;
                    case  "Número":
                        fields[i].CellFormat = "n";
                    break;
                    case "Número sense decimals":
                        fields[i].CellFormat = "n0";
                    break;
                    case "Número amb 4 decimals":
                        fields[i].CellFormat = "n4";
                    break;
                    case "Percentatge":
                        fields[i].CellFormat = "p";
                    break;
                    case "Percentatge sense decimals":
                        fields[i].CellFormat = "p0";
                    break;
                    case "Percentatge amb 4 decimals":
                        fields[i].CellFormat = "p4";
                    break;
                }
                
                i++;
                 
            }

            
            pivotGridControl1.Fields.AddRange(fields);
            pivotGridControl1.EndUpdate(); 
        }

        private void btnGuardar_Click(object sender, RoutedEventArgs e)
        {
            if (!guardat)
            {
                PropietatsCub propietats = new PropietatsCub();
                propietats.Owner = this;
                propietats.txtNom.Text  = nom;
                propietats.txtDescripcio.Text = descripcio;
                propietats.chkTipus.IsChecked = (tipus == 0);

                if (propietats.ShowDialog() != true) 
                    return;

                nom = propietats.txtNom.Text;
                descripcio = propietats.txtDescripcio.Text;
                tipus = (propietats.chkTipus.IsChecked == true) ? 0 : 1;
                rutacompleta = carpeta + "\\" + nom + ".dat";
                guardat = true;

                /*
                if (Utils.InputBox("Nou Cub", "Nom del cub:", ref nomfitxer) == System.Windows.Forms.DialogResult.Cancel)
                {
                    return;
                }
                 */
            }
            pivotGridControl1.SavePivotGridToFile(carpeta + "\\" + nom + ".dat", true); 
        }

        private void btnSortir_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
            Close();
        }

        private void btnImprimir_Click(object sender, RoutedEventArgs e)
        {
            pivotGridControl1.ShowPrintPreview(this);
        }

        private void btnGuardarComA_Click(object sender, RoutedEventArgs e)
        {
            PropietatsCub propietats = new PropietatsCub();
            propietats.Owner = this;
            propietats.txtNom.Text = nom;
            propietats.txtDescripcio.Text = descripcio;
            propietats.chkTipus.IsChecked = (tipus == 0);

            if (propietats.ShowDialog() != true)
                return;

            nom = propietats.txtNom.Text;
            descripcio = propietats.txtDescripcio.Text;
            tipus = (propietats.chkTipus.IsChecked == true) ? 0 : 1;
            rutacompleta = carpeta + "\\" + nom + ".dat";
            guardat = true;
        }
    }
}
