﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.Common;

namespace CubeExplorerUI
{
    class GestorCubs
    {
      
        public GestorCubs()
        {
        }

        public static CubDB CrearCub(int IdSeccio, string carpeta)
        {
            CarregadorDades loader = new CarregadorDades();
            
            // per poder emmagatzemar el registre a la base de dades
            CubDB cubdb = new CubDB();
            OrigensDades od = new OrigensDades(ref loader);
            
            if (od.ShowDialog() != true) return null;
                        
            // Iniciem component de gestió de dades
            GestorDades gd = new GestorDades(ref loader);


            // Sel.lecció de les dades que volem utilitzar
            gd.CarregarDades();

            if (gd.ShowDialog() != true) return null;

            // Element seleccionat
            cubdb.Nom = gd.Element;
            cubdb.IdSeccio = IdSeccio;
            cubdb.Carpeta = carpeta;
            
            // Obtenir la cadena de connexió
            cubdb.Connexio = gd.Connexio;
                       
            // Obtenir el sql sel.leccionat
            cubdb.Sql = gd.Sql;
                                
            // Sel.lecció de camps del cub
            SeleccioCamps sc = new SeleccioCamps(gd.ObtenirDades,cubdb.Camps);

            if (sc.ShowDialog() != true) return null;

      
            // Una vegada definit el format del cub el construïm
            ConstruirCubs cc = new ConstruirCubs();

            cc.nom = cubdb.Nom;  
            cc.Construir(sc.Camps.GetData(), gd.ObtenirDades, carpeta);

            if (cc.ShowDialog() != true) return null;
          
            cubdb.Nom = cc.nom;
            cubdb.Descripcio = cc.descripcio;
            cubdb.Tipus = cc.tipus;
            cubdb.Ruta = cc.rutacompleta;
            cubdb.DataCreacio = DateTime.Now;
            cubdb.DataModificacio = DateTime.Now;
            // Recuperar la llista de camps sel.leccionats del cub
            cubdb.Camps = sc.Camps;

            if (cubdb.Tipus == 0)
            {
                // El cub connectat no emmagatzema aquestes dades
                cubdb.Connexio = "";
                cubdb.Sql = "";
                cubdb.Camps = null;
            }

                       

            // Final creació del cub
            return cubdb;
            
        }

        public static DataSet ObtenirDades(CubDB cubdb)
        {
            DataSet dsConsulta = null;

            // Iniciem component de carrega de dades
            CarregadorDades loader = new CarregadorDades();

            // Connectar a partir de la cadena de connexió
            if (!loader.Connectar(cubdb.Connexio)) return null;

            // Obtenir el dataset a partir de la connexió donada i el sql
            // en el cas de stored procedures cal demanar els valors dels paràmetres
            if (cubdb.Sql.Contains("SP:"))
            {

                string nom = cubdb.Sql.Remove(0, 3);
                // obtenim la col.lecció de paràmetres del procediment emmagatzemat
                DbParameterCollection parameters = loader.GetSpParameters(nom);

                if (parameters.Count > 0)
                {
                    ParametresSP viewSP = new ParametresSP();

                    viewSP.parameters = parameters;

                    // Introduir els paràmetres
                    if (viewSP.ShowDialog() != true)
                    {
                        return null;
                    }

                    // Executar el procediment emmagatzemat amb els paràmetres introduïts
                    dsConsulta = loader.ObtenirDades(nom, parameters);
                    //dsConsulta.WriteXml("C:\\data\\proves.xml");

                }
                else
                {
                    dsConsulta = loader.ObtenirDades(nom, parameters);
                }

            }
            else
            {
                dsConsulta = loader.ObtenirDades(cubdb.Sql);
            }

            loader.Close();
            return dsConsulta;
        }

              
        
        public static bool ModificarCub(ref CubDB cubdb)
        {
            // El cas de cub connectat permet reconstruïr el cub ja que emmagatzema la
            // cadena de connexió de construcció del cub i el SQL de sel.lecció de dades.
            // El cas de cub desconnectat no permet la reconstrucció del cub ni el refresc de
            // les dades.
            if (cubdb.Tipus == 1)
            {
                DataSet dsConsulta = ObtenirDades(cubdb);

                // Sel.lecció de camps del cub
                SeleccioCamps sc = new SeleccioCamps(dsConsulta,cubdb.Camps);

                if (sc.ShowDialog() != true) return false;

                // Una vegada definit el format del cub el construïm
                ConstruirCubs cc = new ConstruirCubs();

                cc.nom = cubdb.Nom;
                cc.descripcio = cubdb.Descripcio;
                cc.tipus = cubdb.Tipus;
                cc.rutacompleta = cubdb.Ruta;
                
                cc.Construir(sc.Camps.GetData(), dsConsulta, cubdb.Carpeta);

                if (cc.ShowDialog() != true) return false;

                cubdb.Nom = cc.nom;
                cubdb.Descripcio = cc.descripcio;
                cubdb.Tipus = cc.tipus;
                cubdb.Ruta = cc.rutacompleta;
                cubdb.DataModificacio = DateTime.Now;
                cubdb.Camps = sc.Camps;


                if (cubdb.Tipus == 0)
                {
                    // El cub connectat no emmagatzema aquestes dades
                    cubdb.Connexio = "";
                    cubdb.Sql = "";
                    cubdb.Camps = null;
                }

                        
                // Final modificació del cub
                return true;
            }
            else
            {
                // Modificació de cub desconnectat
                ConstruirCubs cc = new ConstruirCubs();

                cc.nom = cubdb.Nom;
                cc.descripcio = cubdb.Descripcio;
                cc.tipus = cubdb.Tipus;
                cc.rutacompleta = cubdb.Ruta;

                cc.Construir(cubdb.Carpeta);

                if (cc.ShowDialog() != true) return false;

                cubdb.Nom = cc.nom;
                cubdb.Descripcio = cc.descripcio;
                cubdb.Tipus = cc.tipus;
                cubdb.Ruta = cc.rutacompleta;
                cubdb.DataModificacio = DateTime.Now;
              
              
                return true;
            }
        }
    }
}
