﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using DevExpress.Xpf.Grid;
using System.Data;
using System.Data.Common;

namespace CubeExplorerUI
{
   
    /// <summary>
    /// Lógica de interacción para MainWindow.xaml
    /// </summary>
    public partial class GestorDades : Window
    {
        CarregadorDades loader=null;
        DataSet dsConsulta = null;
        string sql = null;
        DbParameterCollection parameters=null;  // col.lecció de paràmetres del procediment
        string element= null; // element sel.leccionat


        public CarregadorDades Loader
        {
            get { return (loader); }
            set { loader = value; }
        }
        
        public DbParameterCollection Parameters
        {
            get { return parameters; }
        }
        
        public string Sql
        {
            get { return sql; }
        }

        public String Element
        {
            get { return element; }
        }

        public DataSet ObtenirDades {
            get { return (dsConsulta); }
        }

        public string Connexio
        {
            get
            {
                if (loader != null)
                    return loader.Connexio;
                else
                    return "";
            }
        }
        
        public GestorDades(ref CarregadorDades loader)
        {
            this.loader = loader;
            InitializeComponent();
        }

        public void CarregarDades()
        {

           TreeListNode childNode;

       
           if (loader.Taules != null)
           {
               TreeListNode rootNode = CreateRootNode(new ElementBD() { Estructura = "Taules" });
               foreach (string name in loader.Taules)
               {

                   childNode = CreateChildNode(rootNode, new ElementBD() { Estructura = name });
                   childNode.Tag = "Taules";
               }
           }

            
            if (loader.Vistes != null)
            {
                TreeListNode rootNode2 = CreateRootNode(new ElementBD() { Estructura = "Consultes" });
                foreach (string name in loader.Vistes)
                {

                    childNode = CreateChildNode(rootNode2, new ElementBD() { Estructura = name });
                    childNode.Tag = "Vistes";
                }
            }

            if (loader.SP != null)
            {
                TreeListNode rootNode3 = CreateRootNode(new ElementBD() { Estructura = "Stored Procedures" });
                foreach (StoredProcedure sp in loader.SP.Values)
                {

                    childNode = CreateChildNode(rootNode3, new ElementBD() { Estructura = sp.nom  });
                    childNode.Tag = "SP"; 
                }
            }

            this.treeListView1.ShowColumnHeaders = false;
            this.treeListView1.ShowHorizontalLines = false;
            this.treeListView1.ShowVerticalLines = false;
            this.treeListView1.AutoWidth = true;
        }

        private TreeListNode CreateRootNode(object dataObject)
        {
            TreeListNode rootNode = new TreeListNode(dataObject);
            treeListView1.Nodes.Add(rootNode);
            return rootNode;
        }

        private TreeListNode CreateChildNode(TreeListNode parentNode, object dataObject)
        {
            TreeListNode childNode = new TreeListNode(dataObject);
            parentNode.Nodes.Add(childNode);
            return childNode;
        }

        private void btnNext_Click(object sender, RoutedEventArgs e)
        {
            // Identificar si és un procediment emmagatzemat
            TreeListNode node = treeListView1.FocusedNode;
            string tipus = node.Tag.ToString();
            if (tipus != "SP")
            {
                element = treeListView1.GetNodeValue(node, "Estructura").ToString();
                dsConsulta = loader.ObtenirDades(element,ref sql);
                DialogResult = true;
            }
            else
            {
                // Obtenir la col.lecció de paràmetres del procediment emmagatzemat
                element = treeListView1.GetNodeValue(node, "Estructura").ToString();

                // Indiquem a la cadena que es tracta d'un procediment emmagatzemat
                // per poder-ho identificar
                sql = "SP:" + element;
                
                StoredProcedure sp = loader.SP[element];
                
                // obtenim la col.lecció de paràmetres del procediment emmagatzemat
                parameters = loader.GetSpParameters(sp.object_id);

                if (parameters.Count > 0)
                {
                    ParametresSP viewSP = new ParametresSP();

                    viewSP.parameters = parameters;
                    if (viewSP.ShowDialog() == true)
                    {
                        // Executar el procediment emmagatzemat amb els paràmetres introduïts
                        dsConsulta = loader.ObtenirDades(sp.nom, parameters);
                        //dsConsulta.WriteXml("C:\\data\\proves.xml");

                        DialogResult = true;
                        Close();
                    }
                }
                else
                {
                    // Executar el procediment emmagatzemat sense paràmetres
                    dsConsulta = loader.ObtenirDades(sp.nom, parameters);

                    DialogResult = true;
                    Close();
                }
            }
        }

        private void btnSortir_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = false;
            Close();
        }
    }

    public class ElementBD
    {
        public String Estructura { get; set; }
    }
}
