﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.Sql;
using System.Data.SqlClient;
using System.Windows;

namespace CubeExplorerUI
{
    public class Explorador
    {
        string connectionString = Properties.Settings.Default.ConnectionString;
        SqlConnection conn;

        public Explorador() {
            conn = new SqlConnection(connectionString);
        }

        public DataTable LlistarCamps(int Id)
        {
            try {
                // Obrir connexió  
                conn.Open();

                // Llistar tots els camps
                SqlDataAdapter da = new SqlDataAdapter("LlistarCamps", conn);
                da.SelectCommand.CommandType = CommandType.StoredProcedure;
                da.SelectCommand.Parameters.Add ("@Id",SqlDbType.Int);
                da.SelectCommand.Parameters["@Id"].Value = Id;
                        
                DataTable dt = new DataTable();

                da.Fill(dt);

                conn.Close();

                return (dt);
            } 
            catch (Exception e)
            {
                MessageBox.Show(e.Source + ": " + e.Message, "CubeExplorer");
                return null;
            }
        }

              
        public DataTable LlistarCubs(int IdSeccio)
        {
            try {
                // Obrir connexió  
                conn.Open();

                // Llistar tots els cubs
                SqlDataAdapter da = new SqlDataAdapter("[LlistarCubs]", conn);
                da.SelectCommand.CommandType = CommandType.StoredProcedure;
                da.SelectCommand.Parameters.Add("@IdSeccio", SqlDbType.Int);
                da.SelectCommand.Parameters["@IdSeccio"].Value = IdSeccio;
                        
                DataTable dt = new DataTable();
                da.Fill(dt);
                conn.Close();
                
                return (dt);
             } 
            catch (Exception e)
            {
                MessageBox.Show(e.Source + ": " + e.Message, "CubeExplorer");
                return null;
            }
        }


        public void EliminarCamps(int Id)
        {
            try {
                // Obrir connexió  
                conn.Open();

                // ELiminar tots els camps d'un cub
                SqlCommand cmd = new SqlCommand("EliminarCamps", conn);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("@Id", SqlDbType.Int);
                cmd.Parameters["@Id"].Value = Id;

                cmd.ExecuteNonQuery();

                conn.Close();
            } 
            catch (Exception e)
            {
                MessageBox.Show(e.Source + ": " + e.Message, "CubeExplorer");

            }
        }

        public void EliminarCub(int Id)
        {
            try {
                // Obrir connexió  
                conn.Open();

                // ELiminar un cub 
                SqlCommand cmd = new SqlCommand("EliminarCub", conn);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("@Id", SqlDbType.Int);
                cmd.Parameters["@Id"].Value = Id;

                cmd.ExecuteNonQuery();

                conn.Close();
             } 
            catch (Exception e)
            {
                MessageBox.Show(e.Source + ": " + e.Message, "CubeExplorer");

            }
        }

        
                
        
        public void GuardarCamps(int Id, List<FormatCamp> lstCamps)
        {

            try
            {
                conn.Open();

                foreach (FormatCamp fc in lstCamps)
                {

                    // Procediment emmagatzemat de creació dels camps
                    SqlCommand cmd = new SqlCommand("CrearCamps", conn);
                    cmd.CommandType = CommandType.StoredProcedure;

                    SqlParameter p2 = new SqlParameter("@IdCub", Id);
                    p2.SqlDbType = SqlDbType.Int;
                    p2.Direction = ParameterDirection.Input;
                    cmd.Parameters.Add(p2);

                    SqlParameter p3 = new SqlParameter("@Camp", fc.Camp);
                    p3.SqlDbType = SqlDbType.NVarChar;
                    p3.Direction = ParameterDirection.Input;
                    cmd.Parameters.Add(p3);

                    SqlParameter p4 = new SqlParameter("@Area", fc.Area);
                    p4.SqlDbType = SqlDbType.NVarChar;
                    p4.Direction = ParameterDirection.Input;
                    cmd.Parameters.Add(p4);

                    SqlParameter p5 = new SqlParameter("@Titol", fc.Títol);
                    p5.SqlDbType = SqlDbType.NVarChar;
                    p5.Direction = ParameterDirection.Input;
                    cmd.Parameters.Add(p5);

                    SqlParameter p6 = new SqlParameter("@Format", fc.Format);
                    p6.SqlDbType = SqlDbType.NVarChar;
                    p6.Direction = ParameterDirection.Input;
                    cmd.Parameters.Add(p6);


                    cmd.ExecuteNonQuery();
                    cmd.Dispose();

                }
                conn.Close();
            } 
            catch (Exception e)
            {
                MessageBox.Show(e.Source + ": " + e.Message, "CubeExplorer");

            }
        }
        
        public void GuardarCub(ref CubDB cub)
        {
            try { 
                // Procediment emmagatzemat de creació del cub
                SqlCommand cmd = new SqlCommand("CrearCub", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                SqlParameter p1 = new SqlParameter("@Nom", cub.Nom);
                p1.SqlDbType = SqlDbType.NVarChar;
                p1.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p1);

                SqlParameter p2 = new SqlParameter("@ConsultaSql", cub.Sql);
                p2.SqlDbType = SqlDbType.NVarChar;
                p2.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p2);
                        
                SqlParameter p3 = new SqlParameter("@DataCreacio", cub.DataCreacio);
                p3.SqlDbType = SqlDbType.SmallDateTime;
                p3.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p3);

                SqlParameter p4 = new SqlParameter("@DataModificacio", cub.DataModificacio);
                p4.SqlDbType = SqlDbType.SmallDateTime;
                p4.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p4);

                SqlParameter p5 = new SqlParameter("@Descripcio", cub.Descripcio);
                p5.SqlDbType = SqlDbType.NVarChar;
                p5.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p5);

                SqlParameter p6 = new SqlParameter("@Ruta", cub.Ruta);
                p6.SqlDbType = SqlDbType.NVarChar;
                p6.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p6);

                SqlParameter p7 = new SqlParameter("@Tipus", cub.Tipus);
                p7.SqlDbType = SqlDbType.Int;
                p7.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p7);

                SqlParameter p8 = new SqlParameter("@Connexio", cub.Connexio);
                p8.SqlDbType = SqlDbType.NVarChar;
                p8.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p8);

                SqlParameter p9 = new SqlParameter("@IdSeccio", cub.IdSeccio);
                p9.SqlDbType = SqlDbType.Int;
                p9.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p9);
                
                // Id del cub creat
                SqlParameter pRet = new SqlParameter("@RetValue",SqlDbType.Int);
                pRet.Direction = ParameterDirection.ReturnValue;
                cmd.Parameters.Add(pRet);

                conn.Open();
                cmd.ExecuteNonQuery();
                cub.Id = (int)cmd.Parameters["@RetValue"].Value;
                conn.Close ();
                             
             } catch (Exception e) {
                MessageBox.Show  (e.Source + ": " + e.Message, "CubeExplorer"); 
             }  
          }

            
          public void ModificarCub(CubDB cub)
          {
            try { 
                // Obrir connexió  
                conn.Open();

                // Procediment emmagatzemat de creació del cub
                SqlCommand cmd = new SqlCommand("ModificarCub", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                SqlParameter p0 = new SqlParameter("@IdCub", cub.Id);
                p0.SqlDbType = SqlDbType.Int;
                p0.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p0);

                SqlParameter p1 = new SqlParameter("@Nom", cub.Nom);
                p1.SqlDbType = SqlDbType.NVarChar;
                p1.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p1);

                SqlParameter p2 = new SqlParameter("@ConsultaSql", cub.Sql);
                p2.SqlDbType = SqlDbType.NVarChar;
                p2.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p2);
                        
                SqlParameter p3 = new SqlParameter("@DataCreacio", cub.DataCreacio);
                p3.SqlDbType = SqlDbType.SmallDateTime;
                p3.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p3);

                SqlParameter p4 = new SqlParameter("@DataModificacio", cub.DataModificacio);
                p4.SqlDbType = SqlDbType.SmallDateTime;
                p4.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p4);

                SqlParameter p5 = new SqlParameter("@Descripcio", cub.Descripcio);
                p5.SqlDbType = SqlDbType.NVarChar;
                p5.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p5);

                SqlParameter p6 = new SqlParameter("@Ruta", cub.Ruta);
                p6.SqlDbType = SqlDbType.NVarChar;
                p6.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p6);

                SqlParameter p7 = new SqlParameter("@Tipus", cub.Tipus);
                p7.SqlDbType = SqlDbType.Int;
                p7.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p7);

                SqlParameter p8 = new SqlParameter("@Connexio", cub.Connexio);
                p8.SqlDbType = SqlDbType.NVarChar;
                p8.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(p8);

                cmd.ExecuteNonQuery();
          
                conn.Close();

             } catch (Exception e) {
                MessageBox.Show  (e.Source + ": " + e.Message, "CubeExplorer"); 
             }  
           }



            public void ModificarCamps(int Id, List<FormatCamp> lstCamps)
            {

                try
                {
                    // Obrir connexió  
                    conn.Open();


                    foreach (FormatCamp fc in lstCamps)
                    {

                        // Procediment emmagatzemat de creació dels camps
                        SqlCommand cmd = new SqlCommand("ModificarCamps", conn);
                        cmd.CommandType = CommandType.StoredProcedure;

                        SqlParameter p1 = new SqlParameter("@IdCamp", fc.IdCamp);
                        p1.SqlDbType = SqlDbType.Int;
                        p1.Direction = ParameterDirection.Input;
                        cmd.Parameters.Add(p1);
                                                
                        SqlParameter p2 = new SqlParameter("@IdCub", Id);
                        p2.SqlDbType = SqlDbType.Int;
                        p2.Direction = ParameterDirection.Input;
                        cmd.Parameters.Add(p2);
                        
                        SqlParameter p3 = new SqlParameter("@Camp", fc.Camp);
                        p3.SqlDbType = SqlDbType.NVarChar;
                        p3.Direction = ParameterDirection.Input;
                        cmd.Parameters.Add(p3);

                        SqlParameter p4 = new SqlParameter("@Area", fc.Area);
                        p4.SqlDbType = SqlDbType.NVarChar;
                        p4.Direction = ParameterDirection.Input;
                        cmd.Parameters.Add(p4);

                        SqlParameter p5 = new SqlParameter("@Titol", fc.Títol);
                        p5.SqlDbType = SqlDbType.NVarChar;
                        p5.Direction = ParameterDirection.Input;
                        cmd.Parameters.Add(p5);

                        SqlParameter p6 = new SqlParameter("@Format", fc.Format);
                        p6.SqlDbType = SqlDbType.NVarChar;
                        p6.Direction = ParameterDirection.Input;
                        cmd.Parameters.Add(p6);


                        cmd.ExecuteNonQuery();

                    }
                    conn.Close();
                } 
                catch (Exception e)
                {
                    MessageBox.Show(e.Source + ": " + e.Message, "CubeExplorer");
                }
            }
                    
    }
          
}
