﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using DevExpress.Xpf.Core;
using DevExpress.Xpf.Bars;
using DevExpress.Xpf.Layout.Core;
using DevExpress.Xpf.Docking;
using DevExpress.Xpf.NavBar;
using DevExpress.Xpf.Grid;
using DevExpress.Xpf.Printing;
using System.Data; 
using System.Data.SqlClient;
using System.Windows.Forms;
using System.IO;



namespace CubeExplorerUI
{
    enum Mode
    {
        Usuaris=0,
        Cubs=1
    }
    
    // Conectar
    public delegate void ValidarUsuariHandler (string usuari, string clau);
    public delegate void TancarConnexioHandler();

    // Manteniment de usuaris
    public delegate void AceptarUsuariHandler(int id, string usuari, string clau, int perfil);
    public delegate void CancelarUsuariHandler(int id, int mode);

    // Refrescar dades
    public delegate void RefrescDadesHandler(int id);
    
    public partial class MainWindow : Window
    {
        Explorador explorer = new Explorador();
        Usuaris usuaris = new Usuaris();
        Seccions seccions = new Seccions();

        VisorCubs vc;
        int IdSeccio;           // Selecció de la secció activa
        string carpeta;         // Carpeta activa

        bool connectat = false;

        Mode mode;
        System.Data.DataRowView current;

        public void ValidarUsuari(string usuari, string clau)
        {
            // Validació del usuari del sistema
            if (usuaris.ValidarUsuari(usuari, clau))
            {
                connectat = true;
                Configure();

                // Mostrar usuari connectat
                UsuariTancar usClose = new UsuariTancar(usuari);
                usClose.OnTancarConnexio += new TancarConnexioHandler(TancarConnexio);

                UsuariPanel.Content = usClose.layoutTancar;


            }
            else
            {
                System.Windows.Forms.MessageBox.Show("Nom usuari o paraula clau incorrectes.", "CubeExplorer", MessageBoxButtons.OK, MessageBoxIcon.Exclamation); 
            }
        }

        public void TancarConnexio()
        {
            connectat = false;
            Configure();

            UsuariObrir usOpen = new UsuariObrir();

            // Assignar event de validació de usuari
            usOpen.OnValidarUsuari += new ValidarUsuariHandler(ValidarUsuari);

            UsuariPanel.Content = usOpen.layoutObrir;
        }


        public void Configure()
        {
            if (!connectat)
            {
                navBarControl.IsEnabled = false;
                //gridExplorer.ItemsSource = "";
                gridExplorer.Columns.Clear(); 
                gridExplorer.ItemsSource = null;
                nbgNavegador.Items.Clear();  
                VisorPanel.Content = null; 
            } else
            {
                navBarControl.IsEnabled = true;

                // Configuració de permisos de l'aplicació
                // Gestió d'usuaris només administrador
                bool admin = (usuaris.nivell == 1);
                bool dev = (usuaris.nivell == 2);
                bool user = (usuaris.nivell == 3);

                ((NavBarItem)nbgUsuaris.Items["nbiUsuNou"]).IsEnabled = admin;
                ((NavBarItem)nbgUsuaris.Items["nbiUsuMod"]).IsEnabled = admin;
                ((NavBarItem)nbgUsuaris.Items["nbiUsuDel"]).IsEnabled = admin;
                ((NavBarItem)nbgUsuaris.Items["nbiUsuExp"]).IsEnabled = admin;
                // Gestió de seccions només administrador o desenvolupador
                ((NavBarItem)nbgSeccions.Items["nbiSecNou"]).IsEnabled = (admin || dev);
                ((NavBarItem)nbgSeccions.Items["nbiSecDel"]).IsEnabled = (admin || dev);
                // Gestió de cubs només administrador o desenvolupador
                ((NavBarItem)nbgCubs.Items["nbiCubNou"]).IsEnabled = (admin || dev);
                ((NavBarItem)nbgCubs.Items["nbiCubMod"]).IsEnabled = (admin || dev);
                ((NavBarItem)nbgCubs.Items["nbiCubDel"]).IsEnabled = (admin || dev);
                // ((NavBarItem)nbgUsuaris.Items["nbiCubExp"]).IsEnabled = admin && dev;
                
                // Carregar la llista de seccions disponibles
                DataTable dt = seccions.LlistarSeccions();

                if (dt.Rows.Count == 0)
                {
                    // Si no hi ha cap secció creada s'en crea una per defecte
                    string msg = "L'aplicació necessita tenir creada almenys una secció. " + 
                        "Es crearà una secció per defecte";
                    System.Windows.Forms.MessageBox.Show(msg, "CubeExplorer",
                                                         MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                    if (!Directory.Exists("c:\\data\\cubs"))
                        Directory.CreateDirectory("c:\\data\\cubs");
                    
                    seccions.CrearSeccio("Cubs", "Cubs", "c:\\data\\cubs");
                    dt = seccions.LlistarSeccions();
                }


                IdSeccio = 0;
                nbgNavegador.Items.Clear();  
                foreach (DataRow row in dt.Rows) {
                    NavBarItem item = new NavBarItem();
                    item.Name = "sec_" + row[0].ToString();   // Id

                    item.Content = row[1].ToString();       // Nom
                    item.Tag = row[2].ToString();           // Carpeta
                    item.Click += new EventHandler(Seccio_Click);

                    if (IdSeccio == 0)
                    {
                        IdSeccio = Convert.ToInt16(row[0]);
                        carpeta = row[2].ToString();
                    }
                    
                    nbgNavegador.Items.Add(item);
                }

                                        
                // Per defecte mostrar la llista de cubs
                mode = Mode.Cubs;
                // Primera secció per defecte
                gridExplorer.ItemsSource = explorer.LlistarCubs(IdSeccio);
                panelExplorer.Caption = "Explorador: [" + carpeta + "]";
            }

        }

        
        public MainWindow()
        {
            InitializeComponent();

            navBarControl.View.Click += new RoutedEventHandler(View_Click);

            UsuariObrir usOpen = new UsuariObrir();

            // Assignar event de validació de usuari
            usOpen.OnValidarUsuari += new ValidarUsuariHandler(ValidarUsuari);
 
            UsuariPanel.Content = usOpen.layoutObrir;
            
            Configure();   
            
        }

        
        void View_Click(object sender, RoutedEventArgs e)
        {
            NavBarGroup group = navBarControl.View.GetNavBarGroup(e);
            NavBarItem item = navBarControl.View.GetNavBarItem(e);
        }

        private void nbiNou_Click(object sender, EventArgs e)
        {
            // Crear el cub
            CubDB cub = GestorCubs.CrearCub(IdSeccio,carpeta);

            if (cub != null)
            {
                // Donar d'alta el cub a la base de dades
                explorer.GuardarCub(ref cub);
                // Donar d'alta els camps del cub
                if (cub.Camps != null)
                    explorer.GuardarCamps(cub.Id, cub.Camps.GetData()); 
 
                // Refresc de l'explorador
                gridExplorer.ItemsSource = explorer.LlistarCubs(IdSeccio);
                tableView1.MoveLastRow();
            }
        }


        private void nbiEdit_Click(object sender, EventArgs e)
        {
            if (current == null) return;

            CubDB cub = new CubDB();
            
            cub.Id = (int)current.Row.ItemArray[0];
            cub.Nom = (string)current.Row.ItemArray[1];
            cub.Descripcio = (string)current.Row.ItemArray[2];
            cub.Ruta = (string)current.Row.ItemArray[3];
            cub.Tipus = (int)current.Row.ItemArray[4];
            cub.Sql  = (string)current.Row.ItemArray[5];
            cub.Connexio = (string)current.Row.ItemArray[6];
            cub.DataCreacio = (DateTime)current.Row.ItemArray[7];
            cub.DataModificacio = (DateTime)current.Row.ItemArray[8];
            cub.IdSeccio = IdSeccio;
            cub.Carpeta = carpeta;

            // Obtenir els camps del cub
            DataTable dt = explorer.LlistarCamps(cub.Id);
            // Carregar els valors del Data Table
            cub.Camps.SetData(dt);
            
            // Modificació del cub
            if (!GestorCubs.ModificarCub(ref cub)) return;

            // Actualitzar BD
            explorer.ModificarCub(cub);
            if (cub.Camps != null)
                explorer.ModificarCamps(cub.Id,cub.Camps.GetData ());

            // Obtenir el índex de la fila actual
            int index = gridExplorer.GetRowListIndex(tableView1.FocusedRowHandle);
            // Refresc de l'explorador
            gridExplorer.ItemsSource = explorer.LlistarCubs(IdSeccio);

            // Tornar a la posició editada
            tableView1.MoveFocusedRow(index);
        }

        private void nbiEliminar_Click(object sender, EventArgs e)
        {
            if (current == null) return;

            string message = "Vol eliminar el cub " + (string)current.Row.ItemArray[1] + " ?";
            const string caption = "Missatge";
            var result = System.Windows.Forms.MessageBox.Show(message, caption,
                                         MessageBoxButtons.YesNo,
                                         MessageBoxIcon.Question);

            if (result == System.Windows.Forms.DialogResult.Yes)
            {
                explorer.EliminarCub((int)current.Row.ItemArray[0]);
                explorer.EliminarCamps((int)current.Row.ItemArray[0]);
                
                // Refresc de l'explorador
                gridExplorer.ItemsSource = explorer.LlistarCubs(IdSeccio);
            }
        }

        public void OnRefrescarCub(int id)
        {
            CubDB cub = new CubDB();

            cub.Id = (int)current.Row.ItemArray[0];
            cub.Nom = (string)current.Row.ItemArray[1];
            cub.Descripcio = (string)current.Row.ItemArray[2];
            cub.Ruta = (string)current.Row.ItemArray[3];
            cub.Tipus = (int)current.Row.ItemArray[4];
            cub.Sql = (string)current.Row.ItemArray[5];
            cub.Connexio = (string)current.Row.ItemArray[6];
            cub.DataCreacio = (DateTime)current.Row.ItemArray[7];
            cub.DataModificacio = (DateTime)current.Row.ItemArray[8];
            
            DataSet ds = GestorCubs.ObtenirDades(cub);
            vc.Refrescar(ds);
        }
        
        private void tableView1_FocusedRowChanged(object sender, FocusedRowChangedEventArgs e)
        {
            if (mode == Mode.Usuaris) {
                return;
            } else {
                current = (System.Data.DataRowView)e.NewRow;
            }
            

            if (current != null)
            {
                int id = (int)current.Row.ItemArray[0];
                string nom = (string)current.Row.ItemArray[1];
                string ruta = (string)current.Row.ItemArray[3];
                int tipus = (int)current.Row.ItemArray[4];


                vc = new VisorCubs(id, nom, ruta, tipus);
                vc.OnRefrescarDades += new RefrescDadesHandler(OnRefrescarCub);
                VisorPanel.Content = vc.layoutVisor;
            }
            else
            {
                VisorPanel.Content = null;
            }
        }

       
           
        

        private void nbiUsuExp_Click(object sender, EventArgs e)
        {
            mode = Mode.Usuaris;
            gridExplorer.Columns.Clear();
            VisorPanel.Content = null;
            gridExplorer.ItemsSource = usuaris.LlistarUsuaris();
            panelExplorer.Caption = "Usuaris";
           
        }

        private void nbiCubExp_Click(object sender, EventArgs e)
        {
            mode = Mode.Cubs;
            gridExplorer.Columns.Clear(); 
            gridExplorer.ItemsSource = explorer.LlistarCubs(IdSeccio);
            panelExplorer.Caption = "Explorador: [" + carpeta + "]";
            
        }


        public void AceptarUsuari(int id, string usuari, string clau, int perfil)
        {
            navBarControl.IsEnabled = true;
            usuaris.ModificarUsuari(id, usuari, clau, perfil);
            // Refresc dades i UI
            gridExplorer.ItemsSource = usuaris.LlistarUsuaris();
            tableView1.MoveLastRow();
            VisorPanel.Content = null;
        }

        public void CancelarUsuari(int id, int mode)
        {
            navBarControl.IsEnabled = true;
            // El.liminar el registre nou en cas de altes
            if (mode == 1)
            {
                usuaris.EliminarUsuari(id);
                // Refresc dades i UI
                gridExplorer.ItemsSource = usuaris.LlistarUsuaris();
                tableView1.MoveLastRow();
                VisorPanel.Content = null;
            }

        }
        
        
        private void nbiUsuNou_Click(object sender, EventArgs e)
        {
            // Bloquejar UI per evitar reentrada
            navBarControl.IsEnabled = false;
            
            // Crear nou usuari sense dades
            int id = usuaris.CrearUsuari("", "", 3);
            // Refresc dades i UI
            nbiUsuExp_Click(sender, e);
            // Anar al darrer registre
            tableView1.MoveLastRow();

            // Mostrar el últim registre per editar-lo
            UsuarisWnd usuariswnd = new UsuarisWnd();
            usuariswnd.OnAceptarUsuari += new AceptarUsuariHandler(AceptarUsuari);
            usuariswnd.OnCancelarUsuari +=new CancelarUsuariHandler(CancelarUsuari); 
            usuariswnd.id = id;
            usuariswnd.Initialize(1);
            
            VisorPanel.Content = usuariswnd.layoutUsuaris;
        }






        private void nbiUsuMod_Click(object sender, EventArgs e)
        {
            // Bloquejar UI per evitar reentrada
            navBarControl.IsEnabled = false;

            UsuarisWnd usuariswnd = new UsuarisWnd();
            usuariswnd.OnAceptarUsuari += new AceptarUsuariHandler(AceptarUsuari);
            usuariswnd.OnCancelarUsuari += new CancelarUsuariHandler(CancelarUsuari);

            if (mode == Mode.Cubs)
            {
                // Refresc dades i UI
                nbiUsuExp_Click(sender, e);
            }

            DataRow fila = usuaris.ObtenirUsuari(Convert.ToInt32 (gridExplorer.GetFocusedRowCellValue("id")));

            usuariswnd.id = Convert.ToInt32 (fila["Id"]);
            usuariswnd.nom = Convert.ToString(fila["Nom"]);
            usuariswnd.password = Convert.ToString(fila["ParaulaClau"]);
            usuariswnd.perfil = Convert.ToInt32(fila["Perfil"]);

            usuariswnd.Initialize(2);

            VisorPanel.Content = usuariswnd.layoutUsuaris;
        }


        
        private void nbiUsuDel_Click(object sender, EventArgs e)
        {
            if (mode == Mode.Cubs) return;
                        
            DataRow fila = usuaris.ObtenirUsuari(Convert.ToInt32(gridExplorer.GetFocusedRowCellValue("id")));

            string message = "Vol eliminar el usuari " + Convert.ToString(fila["Nom"]) + " ?";
            const string caption = "Missatge";
            var result = System.Windows.Forms.MessageBox.Show(message, caption,
                                         MessageBoxButtons.YesNo,
                                         MessageBoxIcon.Question);

            if (result == System.Windows.Forms.DialogResult.Yes)
            {
                usuaris.EliminarUsuari(Convert.ToInt32(fila["Id"]));
             
                // Refresc de l'explorador
                gridExplorer.ItemsSource = usuaris.LlistarUsuaris();
            }
        }

        private void RefrescSeccio()
        {
            panelExplorer.Caption = "Explorador: [" + carpeta + "]";
       
            mode = Mode.Cubs;
            gridExplorer.Columns.Clear();
            gridExplorer.ItemsSource = explorer.LlistarCubs(IdSeccio);
            tableView1.MoveFirstRow();
        }

        // Selecció de secció activa
        public void Seccio_Click(object sender, EventArgs e)
        {
            NavBarItem item = (NavBarItem)sender;

            // Secció activa
            IdSeccio = Convert.ToInt16(item.Name.Remove(0,4));
            // Carpeta activa
            carpeta = item.Tag.ToString();

            RefrescSeccio();
        }

        private void nbiSecNou_Click(object sender, EventArgs e)
        {
            // Crear nova secció sense dades
            int id = seccions.CrearSeccio("","","");
                                  
            SeccionsWnd seccionswnd = new SeccionsWnd();
            seccionswnd.id = id;
            seccionswnd.Initialize();

            if (seccionswnd.ShowDialog() == true)
            {
                seccions.ModificarSeccio(id, seccionswnd.nom, seccionswnd.descripcio, seccionswnd.carpeta);
                Configure();
            }
        }

        private void nbiSecDel_Click(object sender, EventArgs e)
        {
            string message = "Vol eliminar la secció activa ?";
            const string caption = "Missatge";
            var result = System.Windows.Forms.MessageBox.Show(message, caption,
                                         MessageBoxButtons.YesNo,
                                         MessageBoxIcon.Question);

            if (result == System.Windows.Forms.DialogResult.Yes)
            {
                if (gridExplorer.VisibleRowCount == 0)
                {
                    seccions.EliminarSeccio(IdSeccio);
                    // Refresc de l'explorador
                    Configure();
                }
                else
                {
                    System.Windows.Forms.MessageBox.Show("No es pot el.liminar una secció amb cubs", "CubeExplorer",
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }

            }
        }

        private void Window_Unloaded(object sender, RoutedEventArgs e)
        {
            // Descarrega aplicació
            System.Windows.Application.Current.Shutdown();
        }

           
               
    }
    
}
